/***************************************************************************    
 *  SOIPIX Data calibration class "Calib"
 *                                                                              
 *  File        : Calib.cc                                                    
 *  version     : v 1.1.1 2008/11/30                                             
 *  Author      : Minoru Hirose & Shun Ono                                                
 *                                                                              
 *  Description : class Calib Methods                                         
 *                                                                              
 ****************************************************************************/
#include "calib.h"

Calib::Calib()
{
  Reset();
}


Calib::~Calib(){}

void Calib::Reset()
{
  for(int i=0;i<128;i++)
    for(int j=0;j<128;j++){
      mean[i][j]=0;
      tno[i][j]=0;
      ino[i][j]=0;
      badflg[i][j]=0;
    }
  noiave=0;
  pednum=0;
  noinum=0;
}

void Calib::AddPedData(unsigned short data[][128])
{
  for(int i=0;i<128;i++)
    for(int j=0;j<128;j++){
      mean[i][j] += data[i][j];
    }
  pednum++;
}

void Calib::CalcPed()
{
  if(pednum==0) return;
  for(int i=0;i<128;i++)
    for(int j=0;j<128;j++){
      mean[i][j] /= (double)pednum;
    }
}

void Calib::AddNoiData(unsigned short data[][128])
{
  cms=0;
  for(int i=0;i<128;i++)
    for(int j=0;j<128;j++){ 
      tno[i][j] += ((double)data[i][j]-mean[i][j])*((double)data[i][j]-mean[i][j]);
      cms += data[i][j]-mean[i][j];
    }
  cms /= (double)16384;
  //cout << cms << endl;
  for(int i=0;i<128;i++)
    for(int j=0;j<128;j++){
      ino[i][j] += ((double)data[i][j]-mean[i][j]-cms)*((double)data[i][j]-mean[i][j]-cms);
    }
  noinum++;
}

void Calib::CalcNoi()
{
  if(noinum==0) return;

  for(int i=0;i<128;i++)
    for(int j=0;j<128;j++){
      tno[i][j] = sqrt(tno[i][j]/(double)noinum);
      ino[i][j] = sqrt(ino[i][j]/(double)noinum);
      noiave += ino[i][j]; 
    }
  noiave /= (double)16384;
  double noirms=0;
  for(int i=0;i<128;i++)
    for(int j=0;j<128;j++){
      noirms += (ino[i][j]-noiave)*(ino[i][j]-noiave);
    }
  noirms = sqrt(noirms/(double)16384);
  for(int i=0;i<128;i++)
    for(int j=0;j<128;j++)
      if(ino[i][j] > noiave+noirms){
	//cout << i <<  "\t"  << j<<  "\t:bad channel" << endl;
	badflg[i][j]=1;
      }
}


void Calib::OutPed(double *offset)
{
  memcpy(offset,&mean,sizeof(mean));
}

void Calib::OutTNoise(double *tnoise)
{
  memcpy(tnoise,&tno,sizeof(tno));
}

void Calib::OutINoise(double *inoise)
{
  memcpy(inoise,&ino,sizeof(ino));
}

void Calib::OutBChannel(unsigned short *bch)
{
  memcpy(bch,&badflg,sizeof(badflg));
}
